/*
 * Copyright notice from original mutt:
 * Copyright (C) 1998 Brandon Long <blong@fiction.net>
 * Copyright (C) 1999 Andrej Gritsenko <andrej@lucky.net>
 * Copyright (C) 2000-2002 Vsevolod Volkov <vvv@mutt.org.ua>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef _NNTP_H_
#define _NNTP_H_ 1

#include <lib-sys/mutt_socket.h>
#include <lib-mx/mx.h>

extern mx_t const nntp_mx;

/* number of entries in the hash table */
#define NNTP_CACHE_LEN 10

enum {
    NNTP_NONE = 0,
    NNTP_OK,
    NNTP_BYE
};

typedef struct nntp_data_t nntp_data_t;

typedef struct nntp_server_t {
    unsigned feat_known   : 1;
    unsigned hasXPAT      : 1;
    unsigned hasXGTITLE   : 1;
    unsigned hasXOVER     : 1;
    unsigned hasLISTGROUP : 1;
    unsigned status       : 3;
    char *newsrc;
    char *cache;
    int stat;
    off_t size;
    time_t mtime;
    time_t newgroups_time;
    time_t check_time;
    hash_t *newsgroups;
    nntp_data_t *list;                   /* list of newsgroups */
    CONNECTION *conn;
} nntp_server_t;

typedef struct {
    int   index;
    char *path;
} NNTP_CACHE;

typedef struct {
    int first;
    int last;
} NEWSRC_ENTRY;

struct nntp_data_t {
    struct nntp_data_t *next;

    NEWSRC_ENTRY *entries;
    int num;             /* number of used entries */
    int max;             /* number of allocated entries */
    int unread;
    int firstMessage;
    int lastMessage;
    int lastLoaded;
    int lastCached;
    unsigned subscribed:1;
    unsigned rc:1;
    unsigned new:1;
    unsigned allowed:1;
    unsigned deleted:1;
    char *group;
    char *desc;
    char *cache;
    nntp_server_t *nserv;
    NNTP_CACHE acache[NNTP_CACHE_LEN];
};

DO_INIT(nntp_data_t, nntp_data);
void nntp_data_wipe(nntp_data_t *);
DO_NEW(nntp_data_t, nntp_data);
DO_DELETE(nntp_data_t, nntp_data);
DO_SLIST(nntp_data_t, nntp_data, nntp_data_delete);

nntp_server_t *mutt_select_newsserver (char *);
nntp_data_t *mutt_newsgroup_subscribe (nntp_server_t *, char *);
nntp_data_t *mutt_newsgroup_unsubscribe (nntp_server_t *, char *);
nntp_data_t *mutt_newsgroup_catchup (nntp_server_t *, char *);
nntp_data_t *mutt_newsgroup_uncatchup (nntp_server_t *, char *);
void nntp_clear_cacheindex (nntp_server_t *);
int mutt_newsrc_update (nntp_server_t *);
int nntp_close_mailbox (CONTEXT *);
int nntp_fetch_message (MESSAGE *, CONTEXT *, int);
int nntp_post (const char *);
int nntp_check_msgid (CONTEXT *, const char *);
int nntp_check_children (CONTEXT *, const char *);
int nntp_get_active (nntp_server_t *);
void nntp_buffy (char* dst, ssize_t dstlen);
void nntp_expand_path (char *, ssize_t, ACCOUNT *);
void nntp_logout_all(void);
void nntp_sync_sidebar (nntp_data_t*);

WHERE nntp_server_t *CurrentNewsSrv INITVAL (NULL);

#endif /* _NNTP_H_ */
